package com.cmeeting.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cmeeting.mapper.primary.TecentMeetingMapper;
import com.cmeeting.pojo.TencentMeetingUser;
import com.cmeeting.service.TecentMeetingService;
import com.cmeeting.vo.TencentMeetingVO;
import com.tencentcloudapi.wemeet.Client;
import com.tencentcloudapi.wemeet.core.authenticator.AuthenticatorBuilder;
import com.tencentcloudapi.wemeet.core.authenticator.JWTAuthenticator;
import com.tencentcloudapi.wemeet.core.exception.ClientException;
import com.tencentcloudapi.wemeet.core.exception.ServiceException;
import com.tencentcloudapi.wemeet.service.meetings.api.MeetingsApi;
import com.tencentcloudapi.wemeet.service.meetings.model.V1MeetingsMeetingIdGet200Response;
import com.tencentcloudapi.wemeet.service.meetings.model.V1MeetingsMeetingIdGet200ResponseMeetingInfoListInner;
import com.tencentcloudapi.wemeet.service.records.api.RecordsApi;
import com.tencentcloudapi.wemeet.service.records.model.*;
import lombok.extern.slf4j.Slf4j;
import okhttp3.Headers;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.IOException;
import java.math.BigInteger;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import static com.cmeeting.WeComAndTencentMeeting.fetchUsersInBatches;
import static com.cmeeting.WeComAndTencentMeeting.markDuplicateNamesTecent;

@Service
@Slf4j
public class TecentMeetingServiceImpl extends ServiceImpl<TecentMeetingMapper,TencentMeetingUser> implements TecentMeetingService {
    @Resource
    private TecentMeetingMapper tecentMeetingMapper;

    private static final String HMAC_ALGORITHM = "HmacSHA256";
    private static final char[] HEX_CHAR = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};

    @Value(value = "${tencent.appId}")
    private String tencentAppId;
    @Value(value = "${tencent.sdkId}")
    private String tencentSdkId;
    @Value(value = "${tencent.secretId}")
    private String tencentSecretId;
    @Value(value = "${tencent.secretKey}")
    private String tencentSecretKey;
    @Value(value = "${tencent.admin.userId}")
    private String tencentAdminUserId;

    @Override
    public void batchInsert(List<TencentMeetingUser> users) {
        tecentMeetingMapper.batchInsertUsers(users);
    }


    @Override
    public void doUsers() {
        /**
         * 腾讯会议通过通讯录获取员工信息
         */
        // 1. 构造client客户端
        Client client = new Client.Builder()
                .withAppId(tencentAppId).withSdkId(tencentSdkId)
                .withSecret(tencentSecretId, tencentSecretKey)
                .build();

        // 2.获取到全部用户
        List<TencentMeetingUser> users =  fetchUsersInBatches(client, 3);;
        // 3. 检查重名并设置标志
        markDuplicateNamesTecent(users);

        // 4. 批量插入数据库
        batchInsert(users);
    }

    @Override
    public List<TencentMeetingVO.RecordFile> getMeetingFiles(TencentMeetingVO.TencentMeetingRequest requestVO) {
        StringBuffer url = new StringBuffer();
        url.append("https://api.meeting.qq.com/v1/corp/records?page_size="+requestVO.getPageSize()+"&page="+ requestVO.getPage()
                +"&operator_id="+requestVO.getOperatorId()+"&operator_id_type=1&query_record_type=0");

        // 1.构造 client 客户端(jwt 鉴权需要配置 appId sdkId secretID 和 secretKey)
        Client client = new Client.Builder()
                .withAppId(tencentAppId).withSdkId(tencentSdkId)
                .withSecret(tencentSecretId,tencentSecretKey)
                .build();

        // 2.构造 JWT 鉴权器
        AuthenticatorBuilder<JWTAuthenticator> authenticatorBuilder =
                new JWTAuthenticator.Builder()
                        .nonce(BigInteger.valueOf(Math.abs((new SecureRandom()).nextInt())))
                        .timestamp(String.valueOf(System.currentTimeMillis() / 1000L));

        List<TencentMeetingVO.RecordFile> recordFileUrlList = new ArrayList<>();
        // 3.查询近两天的会议录制列表
        try {
            ZonedDateTime now = ZonedDateTime.now();
            AtomicInteger currentPage = new AtomicInteger(1);
            RecordsApi.ApiV1RecordsGetRequest request =
                    new RecordsApi.ApiV1RecordsGetRequest.Builder()
                            .operatorId(tencentAdminUserId)
                            .operatorIdType("1")
                            .startTime(String.valueOf(now.minusDays(2).toEpochSecond()))
                            .endTime(String.valueOf(now.toEpochSecond()))
                            .pageSize("20")
                            .page(String.valueOf(currentPage.getAndIncrement()))
                            .mediaSetType("0")
                            .queryRecordType("0")
                            .build();
            RecordsApi.ApiV1RecordsGetResponse response =
                    client.records().v1RecordsGet(request, authenticatorBuilder);
            V1RecordsGet200Response data = response.getData();
            if (data != null && data.getRecordMeetings() != null && !data.getRecordMeetings().isEmpty()) {
                List<V1RecordsGet200ResponseRecordMeetingsInner> meetings = data.getRecordMeetings();

                //录制状态：
                //1：录制中
                //2：转码中
                //3：转码完成
                if(meetings.stream().allMatch(item->item.getState() != 3)){
                    return null;
                }
                for (V1RecordsGet200ResponseRecordMeetingsInner meeting : meetings) {
                    if(meeting.getState() != 3) continue;

                    //查询会议详情
                    String meetingId = meeting.getMeetingId();
                    String subMeetingId = null;
                    MeetingsApi.ApiV1MeetingsMeetingIdGetRequest meetingRequest =
                            new MeetingsApi.ApiV1MeetingsMeetingIdGetRequest.Builder(meetingId)
                                    .operatorId(tencentAdminUserId)
                                    .operatorIdType("1")
                                    .instanceid("0")
                                    .build();
                    try {
                        MeetingsApi.ApiV1MeetingsMeetingIdGetResponse meetingResponse =
                                client.meetings().v1MeetingsMeetingIdGet(meetingRequest, new JWTAuthenticator.Builder()
                                        .nonce(BigInteger.valueOf(Math.abs((new SecureRandom()).nextInt())))
                                        .timestamp(String.valueOf(System.currentTimeMillis() / 1000L)));
                        V1MeetingsMeetingIdGet200Response meetingResponseData = meetingResponse.getData();
                        List<V1MeetingsMeetingIdGet200ResponseMeetingInfoListInner> meetingInfoList = meetingResponseData.getMeetingInfoList();
                        //尝试获取会议详情
                        if(meetingInfoList != null && meetingInfoList.size() > 0){
                            V1MeetingsMeetingIdGet200ResponseMeetingInfoListInner meetingInfo = meetingInfoList.get(0);
                            //会议类型。
                            //0：一次性会议
                            //1：周期性会议
                            Long meetingType = meetingInfo.getMeetingType();
                            if(meetingType.intValue() == 1){
                                subMeetingId = meetingInfo.getCurrentSubMeetingId();
                            }
                        }
                    } catch (ClientException e) {
                        throw new RuntimeException(e);
                    } catch (ServiceException e) {
                        throw new RuntimeException(e);
                    }

                    List<V1RecordsGet200ResponseRecordMeetingsInnerRecordFilesInner> recordFiles = meeting.getRecordFiles();
                    for (V1RecordsGet200ResponseRecordMeetingsInnerRecordFilesInner recordFile : recordFiles) {
                        TencentMeetingVO.RecordFile recordFileItem = TencentMeetingVO.RecordFile.builder()
                                .recordFileId(recordFile.getRecordFileId())
                                .meetingId(meetingId).subMeetingId(subMeetingId).build();
                        recordFileUrlList.add(recordFileItem);
                    }
                }
            }

        } catch (Exception e) {
            log.error(e.getMessage());
            throw new RuntimeException(e.getMessage());
        }
        return recordFileUrlList;
    }

    /**
     * 生成请求腾会接口的请求头
     * @param httpMethod http请求方法 GET/POST/PUT等
     * @param requestUri 请求uri，eg：/v1/meetings
     * @return
     */
    private Headers generateHeaders(String httpMethod, String requestUri){
        //生成签名的时间戳
        String headerTimestamp = String.valueOf(LocalDateTime.now().atZone(ZoneId.systemDefault()).toEpochSecond());
        //生成随机正整数(暂定八位
        String randomNumber = String.valueOf((int)(Math.random() * 99999999) + 1);

        //生成签名
        String signature = sign(tencentSecretId, tencentSecretKey, httpMethod, randomNumber, headerTimestamp, requestUri, "{}");
        Headers headers = new Headers.Builder()
                .add("Content-Type", "application/json")
                .add("X-TC-Key", tencentSecretId)
                .add("X-TC-Timestamp", headerTimestamp)
                .add("X-TC-Nonce", randomNumber)
                .add("X-TC-Signature",signature)
                .add("AppId",tencentAppId)
                .add("SdkId",tencentSdkId)
                .add("X-TC-Registered","1")
                .build();
        return headers;
    }

    /**
     * 生成签名，开发版本oracle jdk 1.8.0_221
     *
     * @param secretId 邮件下发的secret_id
     * @param secretKey 邮件下发的secret_key
     * @param httpMethod http请求方法 GET/POST/PUT等
     * @param headerNonce X-TC-Nonce请求头，随机数
     * @param headerTimestamp X-TC-Timestamp请求头，当前时间的秒级时间戳
     * @param requestUri 请求uri，eg：/v1/meetings
     * @param requestBody 请求体，没有的设为空串
     * @return 签名，需要设置在请求头X-TC-Signature中
     * @throws NoSuchAlgorithmException e
     * @throws InvalidKeyException e
     */
    private static String sign(String secretId, String secretKey, String httpMethod, String headerNonce, String headerTimestamp, String requestUri, String requestBody) {
        String tobeSig =
                httpMethod + "\nX-TC-Key=" + secretId + "&X-TC-Nonce=" + headerNonce + "&X-TC-Timestamp=" + headerTimestamp + "\n" + requestUri + "\n" + requestBody;
        Mac mac;
        try {
            mac = Mac.getInstance(HMAC_ALGORITHM);
            SecretKeySpec secretKeySpec = new SecretKeySpec(secretKey.getBytes(StandardCharsets.UTF_8), mac.getAlgorithm());
            mac.init(secretKeySpec);
        } catch (Exception e) {
            log.error("获取腾会签名错误："+e.getMessage());
            throw new RuntimeException("请求失败");
        }
        byte[] hash = mac.doFinal(tobeSig.getBytes(StandardCharsets.UTF_8));
        String hexHash = bytesToHex(hash);
        String signature = new String(Base64.getEncoder().encode(hexHash.getBytes(StandardCharsets.UTF_8)));
        return signature;
    }

    private static String bytesToHex(byte[] bytes) {

        char[] buf = new char[bytes.length * 2];
        int index = 0;
        for (byte b : bytes) {
            buf[index++] = HEX_CHAR[b >>> 4 & 0xf];
            buf[index++] = HEX_CHAR[b & 0xf];
        }
        return new String(buf);
    }
}
